
%MiceTemplates Project

%DELETING CAROLINE...GOODBYE CAROLINE
clear all;
close all;

%Point 0. S = slow wave activity vector, G=group, A=age, E=exercise

%Point 1. We want to further quantify (and solidify) the feasibility of SWA as a
%biomarker of age and exercise.

%Point 2. The preprocessing is the same to all groups/mice i.e. same
%dimensionality reduction and same parameter estimation techniques

%Point 3. Our focal point is on the confidence and stability of the
%results, i.e. we want to guarantee that SWA signals provide information
%about the age and exercise of mice

%Point 4. In order to best capture the informativeness of EEG signals we
%investigate profiled techniques (templating), i.e. supervised machine
%learning. 

%Point 5. Part of the observations are used to estimate a probabilistic
%model Pr_{model}[G=g | S=s ]. Then, we can use the model 1. to extract
%information about group G = (A,E) and 2. determine the success rate of SWA
%as a biomarker

%Point 6. Repeating: First, we use information-theoretic analysis to demonstrate that there
%exists information in S about G, i.e. solidify S as a biomarker of age and
%exercise. Second, we analyze using the success rate to determine how easy
%is it to extract this information using S. We consider both types
%of analyses in our experiments because the first one allow a better assessment of
%the confidence  while the second one leads to simpler intuitions regarding the concrete
%effectiveness of the biormarker.


%Point 7. In the information theoretic analysis we use perceived
%information PI(G;S), which quantifies the amount of information for group G
%Since our model (multivariate PDF estimation) is never the true
%distribution, the PI can capture the amount of information that is
%extracted from S, biased by the assumption and estimation errors of our
%model


%Data 1. Every group has n_g vectors of slow way activity of dimension d
no_groups=6; %number of groups
no_mice = [11 9 8 9 9 8]; %number of mice per group

%Data 0. Import dataset
sdim=12; %swa vector dimension
poi=1:sdim;
load('C:\MatlabScripts\MiceTemplate\dataset\swa_1hour.mat');
for i=1:no_groups
    S{i}=g{i}(1:no_mice(i),poi);
%     %Remove NaN via interpolation? Nah, there are issues with it
%     for j=1:no_mice(i)
%         S{i}(j,:)=naninterp(S{i}(j,:));
%     end
end

%Data 1. Reduce classes by grouping old mice together

S4{1}=S{1};S4{2}=S{2};
S4{3}=vertcat(S{3},S{5});
S4{4}=vertcat(S{4},S{6});
%new group sizes
no_mice4 = [ 11 9 17 17];
no_groups4 =4;

%Data 11. Reduce to 3 classes by grouping oldexercise group together with
%young control group due to their close similarities
S3{1}=S{2}; %young exercise 
S3{2}= vertcat(vertcat(S{4},S{6}),S{1}); %old exercise + young control
S3{3}=vertcat(S{3},S{5}); %old control
%new group sizes
no_mice3 = [ 9 28 17];
no_groups3 =3;

%Classification choice
%class= 1 for 6-group classification
%class= 2 for 4-group classification
%class= 3 for 3-group classification

class=1;
if (class==2)
    clear S;
    S=S4;
    no_mice=no_mice4;
    no_groups=no_groups4;
end
if (class==3)
    clear S;
    S=S3;
    no_mice=no_mice3;
    no_groups=no_groups3;
end


%Data 2. Simulate dataset, just for testing the code
%e.g. Initialize to zeros in cell array
%S={zeros(no_mice(1),sdim), zeros(no_mice(2),sdim), zeros(no_mice(3),sdim),zeros(no_mice(4),sdim)};
%e.g. Simulate data from multivariate normal
% for i=1:no_groups
%     sim_me=repmat(i,1,sdim);
%     sim_C = 0.5*eye(sdim);
%     sim_swa = mvnrnd(sim_me,sim_C,no_mice(i));
%     S{i}=sim_swa;
% end


%Code 1. Proving the applicability of SWA as a biomarker
%Cross validation of PI


    
%Data 4a. Leave-one-out partition as specified in PIN paper   
    
for m=3:3 %number of POIs
     
    n=sum(no_mice); %leave one out technique
    counter=0;
    for group=1:no_groups
        for mouse=1:size(S{group},1)
            
            S_test=S{group}(mouse,:); %leave-one-out test set          
            S_train{group}=S{group}(setdiff(1:size(S{group},1),mouse),:);
            for i=setdiff(1:no_groups,group) %create train set, excluding the test set
                S_train{i}=S{i};
            end
                
            counter=counter+1;
            [perceived_info(counter) RS2]=PI(S_train,S_test,no_groups,sdim,m,group);        
        end
    end
end
       

mPI=mean(perceived_info)
stdPI=std(perceived_info)           



%TEST silouette plots

% cl = zeros(53,2);
% counter=1;
% for i=1:no_groups
%     for j=1:size(RS2{i},1)
%         cl(counter,:) = RS2{i}(j,:)
%         counter=counter+1;
%     end
% end
% for k=6:6
%     [cidx2,cmeans2] = kmeans(cl,k,'dist','sqeuclidean');
%     figure;
%     [silh2,h] = silhouette(cl,cidx2,'sqeuclidean');
%     score(k) = mean(silh2);
% end

%pca-reduced cluster
figure;
for i=1:no_groups
     scatter(RS2{i}(:,1),RS2{i}(:,2),'filled');
     
     %surf(RS2{i}(:,1),RS2{i}(:,2),RS2{i}(:,3))
     %scatter(cmeans2(i,1),cmeans2(i,2));
     hold on;
end

%compute center of gravity
%figure;
for i=1:no_groups
     %scatter3(RS2{i}(:,1),RS2{i}(:,2),RS2{i}(:,3),'filled');
     mesi(i,:)=mean(RS2{i});
     
     hold on;
end

%compute class distance

for i=1:no_groups
    for j=1:no_groups
        distance(i,j)=norm(mesi(i,:)-mesi(j,:),2);
    end
end


figure;
for i=1:no_groups
     scatter3(mesi(i,1),mesi(i,2),mesi(i,3),'filled');
     %mesi(i,:)=mean(RS2{i});
     %surf(RS2{i}(:,1),RS2{i}(:,2),RS2{i}(:,3))
     %scatter(cmeans2(i,1),cmeans2(i,2));
     hold on;
end
% figure;
% for i=1:no_groups
%      scatter3((RS2{i}(:,1)),(RS2{i}(:,2)),(RS2{i}(:,3)),'filled');
%      hold on;
%     % scatter3(mesi(i,1),mesi(i,2),mesi(i,3),90,'x','MarkerEdgeColor',[0 0 0],...
%        %       'MarkerFaceColor',[0 0 0],...
%         %      'LineWidth',3.5);
%    % hold on;
%      
% end


% hold off;
% lgd = legend('young control','young exercise','18m control','18m exercise','24m control','24m exercise');
% title(lgd,'Mice Groups')

% %A random approach to cross-validation (not very useful, still open questions here)        
% for m=2:2 %number of POIs
%         B=100;
%         for counter=1:B     
%       
    %Data 4b. Leave-one-out partition randomly IN TOTAL
    
%            out_g =randi(no_groups);%choose group randomly
%            temp=S{out_g}; 
%            out=randi(size(temp,1)); %choose mouse randomly - not sure if double rand choice is appropriate
%            train_vector=setdiff(1:size(temp,1),out);
%            S_train{out_g}=temp(train_vector,:);
%            S_test=temp(out,:); %the one-valued test set
%               
%          for i=setdiff(1:no_groups,out_g) 
%              S_train{i}=S{i};
%          end
        
%     %Data 4c. Leave-one-out partition randomly PER CLASS
%         for i=1:no_groups   
%             temp=S{i};
%             out=randi(size(temp,1));
%             train_vector=setdiff(1:size(temp,1),out);
%             %S_train{i}=temp(1:(size(temp,1)-1),:);
%             S_train{i}=temp(train_vector,:);
%             S_test{i}=temp(out,:);
%         end
        %Compute PI
%         perceived_info(counter)=PI(S_train,S_test,no_groups,sdim,m,out_g);
%         end
% end
    

%sampling on mean PI, computing bootstrap error
for B=1:100 %number of bootstrap samples
    rs=randi(size(perceived_info,2),54,1);
    pi_br(B)=mean(perceived_info(rs));
end
percentile95=pi_br(95);
percentile5=pi_br(5);



%Code 2. Success rate of SWA as a biomarker
%50-50 train/test ratio


partition_vec=linspace(0.5,0.5,1);
p_counter=1;
for p=partition_vec
    for i=1:no_groups
        t=ceil(p*size(S{i},1));
        train_vec{i}= 1:t;
        test_vec{i} = (t+1):size(S{i},1);
    end

    for m=2:6
        success_rate(p_counter,m,:)=SR(S,sdim,train_vec,test_vec,m);
        squeeze(success_rate(p_counter,m,:));
    end
    p_counter=p_counter+1;
end
 sr=squeeze(sum(success_rate,3));

 
 %Code 3. Clustering
 
 unsupervised_dataset = [S{1};S{2};S{3};S{4};S{5};S{6}];
coef= pca(unsupervised_dataset);
reduced_unsupervised_dataset = unsupervised_dataset * coef(:,1:3);


%GapStatistic clustering 
 %res=evalclusters(unsupervised_dataset,'kmeans','Gap','B',100,'Distance','sqEuclidean','KList',[1:6])
 res2=evalclusters(reduced_unsupervised_dataset,'kmeans','Gap','B',100,'Distance','sqEuclidean','KList',[1:6])

 for b=1:100
        
        %[cluster_indexes(:,b) C sumd ]= kmeans(unsupervised_dataset,6,'Distance','sqEuclidean','MaxIter',1000);
        [cluster_indexes(:,b) C sumd ]= kmeans(reduced_unsupervised_dataset,6,'Distance','sqEuclidean','MaxIter',1000);
        %[hh(:,b) u(:,b)]=hist(cluster_indexes(:,b),unique(cluster_indexes(:,b)));
    if (b==1)
        oldsumd=sum(sumd); %init
        best_b=1;
        best_C=C;
    end
    if (sum(sumd)<oldsumd)
        best_b=b;
        best_C=C;
    end
    
   
 end
 
 b=best_b;
  %mis-clustering rates
    r1=sort(histc(cluster_indexes(1:11,b),unique(cluster_indexes(1:11,b))),'descend');
    mc1=r1(1)/sum(r1);
    r2=sort(histc(cluster_indexes(12:20,b),unique(cluster_indexes(12:20,b))),'descend');
    mc2=r2(1)/sum(r2);
    r3=sort(histc(cluster_indexes(21:28,b),unique(cluster_indexes(21:28,b))),'descend');
    mc3=r3(1)/sum(r3);
    r4=sort(histc(cluster_indexes(29:37,b),unique(cluster_indexes(29:37,b))),'descend');
    mc4=r4(1)/sum(r4);
    r5=sort(histc(cluster_indexes(38:46,b),unique(cluster_indexes(38:46,b))),'descend');
    mc5=r5(1)/sum(r5);
    r6=sort(histc(cluster_indexes(47:54,b),unique(cluster_indexes(47:54,b))),'descend');
    mc6=r6(1)/sum(r6);
 
 miscluster_rates=[mc1 mc2 mc3 mc4 mc5 mc6]
 
 %%
 figure;
for i=1:no_groups
    scatter3((RS2{i}(:,1)),(RS2{i}(:,2)),(RS2{i}(:,3)),'filled');  
             
    
     
    
    %legend, 'Young Control','Young Exercise','18m Control','18m Exercise','24 m Control','24m Exercise'
      hold on;
 
                     
end
%   
for i=1:no_groups
     scatter3(mesi(i,1),mesi(i,2),mesi(i,3),90,'x','MarkerEdgeColor',[0 0 0],...
             'MarkerFaceColor',[0 0 0],...
             'LineWidth',3.5); 
end
hold off;
legend('Young Control','Young Exercise','18m Control', '18m Exercise', '24 m Control', '24m Exercise', 'FontSize',12)  
xlabel('x axis (a.u.)', 'FontSize',18)
ylabel('y axis (a.u.)', 'FontSize',18)
zlabel('z axis (a.u.)', 'FontSize',18)

